package example;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInput;

import step.*;
import step.io.*;

import example.jvmpi.*; // events

public class AdaptToStep
{
  public static final int BUF_SIZE = 65536;

  public static void main(String[] args)
  {
    try
    {
      InputStream is = (args.length >= 1) ?
        (InputStream) new java.io.FileInputStream(args[0]) :
        (InputStream) System.in;

      DataInput input =
        new java.io.DataInputStream(
        new java.io.BufferedInputStream(is, BUF_SIZE));

      OutputStream os = (args.length >= 1) ?
        (OutputStream) new java.io.FileOutputStream(args[0]+".step") :
        (OutputStream) System.out;

      StepRecordOutput stepOut =
        new StepEncodedOutput(
        new java.io.BufferedOutputStream(os, BUF_SIZE));

      int magic = input.readInt();
      byte[] options = new byte[111];
      input.readFully(options);

      stepOut.writeRecord(new AdaptHeader(new StepInt(magic), new StepData(options)));

      long count = 0L;
      try
      {
        while (true)
        {
          StepRecord record;

          byte event = input.readByte();

          switch (event)
          {
            case 0x00:
              record = new ARENA_DELETE(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:arenaId*/)
              );
              break;

            case 0x01:
              {
                int envId = input.readInt();
                int arenaId = input.readInt();
                short len = input.readShort();
                byte[] bytes = new byte[len];
                input.readFully(bytes);
                record = new ARENA_NEW(
                  new StepInt(envId),
                  new StepInt(arenaId),
                  new StepString(new String(bytes))
                );
              }
              break;

            case 0x09:
              record = new GC_FINISH(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readLong()/*:usedObjects*/),
                new StepInt(input.readLong()/*:usedObjectSpace*/),
                new StepInt(input.readLong()/*:usedTotalSpace*/)
              );
              break;

            case 0x0A:
              record = new GC_START(new StepInt(input.readInt()/*:envId*/));
              break;

            case 0x10:
              record = new JVM_INIT_DONE(new StepInt(input.readInt()/*:envId*/));
              break;

            case 0x11:
              record = new JVM_SHUT_DOWN(new StepInt(input.readInt()/*:envId*/));
              break;

            case 0x1B:
              record = new OBJECT_ALLOC(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:arenaId*/),
                new StepInt(input.readInt()/*:classId*/),
                new StepInt(input.readInt()/*:isArray*/),
                new StepInt(input.readInt()/*:size*/),
                new StepInt(input.readInt()/*:objId*/)
              );
              break;

            case 0x1D:
              record = new OBJECT_FREE(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:objId*/)
              );
              break;

            case 0x13:
              record = new METHOD_ENTRY2(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:methodId*/),
                new StepInt(input.readInt()/*:objId*/)
              );
              break;

            case 0x14:
              record = new METHOD_EXIT(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:methodId*/)
              );
              break;

            case 0x24:
              record = new INSTRUCTION_START(
                new StepInt(input.readInt()/*:envId*/),
                new StepInt(input.readInt()/*:methodId*/),
                new StepInt(input.readInt()/*:fieldOffset*/),
                new StepInt(input.readInt()/*:fieldIsTrue*/),
                new StepInt(input.readInt()/*:fieldKey*/),
                new StepInt(input.readInt()/*:fieldLow*/),
                new StepInt(input.readInt()/*:fieldHigh*/),
                new StepInt(input.readInt()/*:pairIndex*/),
                new StepInt(input.readInt()/*:pairTotal*/)
              );
              break;

            case 0xFFFFFFEF: // FILE SPLIT
              return;
              
            default:
              throw new java.io.IOException("unknown event: "+event);
          }

          stepOut.writeRecord(record);
          count++;

          if ( ( count % 10000L ) == 0 )
            System.err.print('.');
        }
      }
      catch (java.io.EOFException e)
      {
        // OK
      }
      finally
      {
        System.err.println();
        System.err.println("encoded "+count+" events");

        stepOut.flush();
        stepOut.close();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
}
