package example;

// Java Imports
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// Soot Imports
import soot.*;
import soot.jimple.*;

/** A Jimple transformer that inserts code to profile object allocations. */
public class AllocationProfiler extends BodyTransformer
{
  protected void internalTransform(Body body, String phaseName, Map options)
  {
    // identify each new site with an unique ID
    int siteNum = 1;

    // get a modifiable list of the method statements
    PatchingChain units = body.getUnits();

    // look for new expressions
    Iterator iUnits = units.snapshotIterator();
    while (iUnits.hasNext())
    {
      Unit unit = (Unit) iUnits.next();

      if (unit instanceof DefinitionStmt)
      {
        DefinitionStmt stmt    = (DefinitionStmt) unit;
        Type           newType = null;

        if (stmt.getRightOp() instanceof NewExpr)
        {
          newType = ((NewExpr) stmt.getRightOp()).getType();
        }
        else if (stmt.getRightOp() instanceof NewArrayExpr)
        {
          newType = ((NewArrayExpr) stmt.getRightOp()).getType();
        }
        else if (stmt.getRightOp() instanceof NewMultiArrayExpr)
        {
          newType = ((NewMultiArrayExpr) stmt.getRightOp()).getType();
        }
        else // => not an allocation
          continue;

        // register this creation with the collector
        {
          List args = new java.util.ArrayList(3);
          args.add(StringConstant.v(newType.toString()));
          args.add(StringConstant.v(body.getMethod().getSignature()));
          args.add(IntConstant.v(siteNum++));

          // collector.allocation(args)
          units.insertAfter(
            Jimple.v().newInvokeStmt(Jimple.v().newVirtualInvokeExpr(
              Profiler.getCollectorLocal(),
              Profiler.getCollectorClass().getMethodByName("allocation"),
              args)),
            unit);
        }
      }
    }
  }
}
