package example;

// Java Imports
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// Soot Imports
import soot.*;
import soot.jimple.*;

/** A Jimple transformer that inserts code to profile field usage. */
public class FieldAccessProfiler extends BodyTransformer
{
  protected void internalTransform(Body body, String phaseName, Map options)
  {
    // use an int ID for each access site
    int siteNum = 1;

    // get a modifiable list of the method statements
    PatchingChain units = body.getUnits();

    // look for field accesses
    Iterator iUnits = units.snapshotIterator();
    while (iUnits.hasNext())
    {
      Unit unit = (Unit) iUnits.next();

      if (unit instanceof DefinitionStmt)
      {
        DefinitionStmt stmt  = (DefinitionStmt) unit;
        SootField      field = null;
        boolean        read  = false;

        if (stmt.getLeftOp() instanceof FieldRef) // => field write
        {
          field = ((FieldRef) stmt.getLeftOp()).getField();
          read  = false;
        }
        else if (stmt.getRightOp() instanceof FieldRef) // => field read
        {
          field = ((FieldRef) stmt.getRightOp()).getField();
          read  = true;
        }
        else // => not a field access
          continue;

        // register this access with the collector
        {
          List args = new java.util.ArrayList(7);
          args.add(StringConstant.v(field.getName()));
          args.add(StringConstant.v(field.getType().toString()));
          args.add(StringConstant.v(field.getDeclaringClass().getType().toString()));
          args.add(IntConstant.v(Modifier.isStatic(field.getModifiers()) ? 1 : 0));
          args.add(IntConstant.v((read) ? 1 : 0));
          args.add(StringConstant.v(body.getMethod().getSignature()));
          args.add(IntConstant.v(siteNum++));

          // collector.fieldAccess(args)
          units.insertAfter(
            Jimple.v().newInvokeStmt(Jimple.v().newVirtualInvokeExpr(
              Profiler.getCollectorLocal(),
              Profiler.getCollectorClass().getMethodByName("fieldAccess"),
              args)),
            stmt);
        }
      }
    }
  }
}
