package example;

// Java Imports
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// Soot Imports
import soot.*;
import soot.jimple.*;

/**
 * A Jimple transformer that inserts code to profile method enters and exits.
 * <p>
 * NOTE: Exceptional exits are not tracked.
 */
public class MethodProfiler extends BodyTransformer
{
  protected void internalTransform(Body body, String phaseName, Map options)
  {
    // identify separate exit points with a separate ID
    int siteNum = 1;

    // get a modifiable list of the method statements
    PatchingChain units = body.getUnits();

    // collector.methodEnter("method signature")
    units.insertAfter(
      Jimple.v().newInvokeStmt(Jimple.v().newVirtualInvokeExpr(
        Profiler.getCollectorLocal(),
        Profiler.getCollectorClass().getMethodByName("methodEnter"),
        StringConstant.v(body.getMethod().getSignature()))),
      units.getFirst());

    // look for exit points
    // *** NOTE: This will not catch exit due to exceptions! *** 
    Iterator iUnits = units.snapshotIterator();
    while (iUnits.hasNext())
    {
      Unit unit = (Unit) iUnits.next();

      if ((unit instanceof ReturnStmt) || (unit instanceof ReturnVoidStmt))
      {
        // collector.methodExit("method signature", exitId)
        units.insertBefore(Jimple.v().newInvokeStmt(Jimple.v().newVirtualInvokeExpr(
            Profiler.getCollectorLocal(),
            Profiler.getCollectorClass().getMethodByName("methodExit"),
            StringConstant.v(body.getMethod().getSignature()),
            IntConstant.v(siteNum++))),
          unit);
      }
    }
  }
}
