package example;

import java.util.Map;

import soot.*;
import soot.jimple.*;

public class Profiler extends BodyTransformer
{
  /** Add profilers to the Jimple Transformation Pack, then invoke Soot's main(). */
  public static void main(String [] args)
  {
    Scene.v().getPack("jtp").add(
      new Transform("jtp.Profiler.prepass", new Profiler()));

    Scene.v().getPack("jtp").add(
      new Transform("jtp.AllocationProfiler", new AllocationProfiler()));

    Scene.v().getPack("jtp").add(
      new Transform("jtp.MethodProfiler", new MethodProfiler()));

    Scene.v().getPack("jtp").add(
      new Transform("jtp.FieldAccessProfiler", new FieldAccessProfiler()));

    soot.Main.main(args);
  }

  private static SootClass _ProfileCollectorClass;

  public static SootClass getCollectorClass()
  {
    if ( _ProfileCollectorClass == null )
      _ProfileCollectorClass =
        Scene.v().loadClassAndSupport("example.ProfileCollector");

    return _ProfileCollectorClass;
  }

  private static Local _ProfileCollectorLocal;

  public static Local getCollectorLocal()
  {
    if ( _ProfileCollectorLocal == null )
      _ProfileCollectorLocal =
         Jimple.v().newLocal("profileCollector", getCollectorClass().getType());

    return _ProfileCollectorLocal;
  }

  protected void internalTransform(Body body, String phaseName, Map options)
  {
    body.getLocals().add(getCollectorLocal());

    body.getUnits().insertBefore(
      Jimple.v().newAssignStmt(getCollectorLocal(),
        Jimple.v().newStaticInvokeExpr(getCollectorClass().getMethodByName("v"))),
      body.getUnits().getFirst());
  }
}
