package example;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataOutputStream;

import step.*;
import step.io.*;

import example.jvmpi.*;

public class StepToAdapt
{
  public static final int BUF_SIZE = 65536;

  public static void main(String[] args)
  {
    try
    {
      InputStream is = (args.length >= 1) ?
        (InputStream) new java.io.FileInputStream(args[0]) :
        (InputStream) System.in;

      StepRecordInput stepIn =
        new StepEncodedInput(
        new java.io.BufferedInputStream(is, BUF_SIZE));

      OutputStream os = (args.length >= 1) ?
        (OutputStream) new java.io.FileOutputStream(args[0]+".dat") :
        (OutputStream) System.out;

      DataOutputStream output =
        new java.io.DataOutputStream(
        new java.io.BufferedOutputStream(os, BUF_SIZE));

      AdaptHeader header = (AdaptHeader) stepIn.readRecord();

      output.writeInt(header.magic.intValue());
      output.write(header.options.getData());
      
      long count = 0L;
      try
      {
        while (true)
        {
          StepRecord record = stepIn.readRecord();

          if ( record instanceof ARENA_DELETE )
          {
            ARENA_DELETE r = (ARENA_DELETE) record;

            output.writeByte(0x00); // ADAPT_ARENA_DELETE

            output.writeInt(r.envId.intValue());
            output.writeInt(r.arenaId.intValue());
          }
          else if ( record instanceof ARENA_NEW )
          {
            ARENA_NEW r = (ARENA_NEW) record;

            output.writeByte(0x01); // ADAPT_ARENA_NEW

            output.writeInt(r.envId.intValue());
            output.writeInt(r.arenaId.intValue());
            output.writeShort(r.arenaName.toString().length());
            output.write(r.arenaName.toString().getBytes());
          }
          else if ( record instanceof GC_FINISH )
          {
            GC_FINISH r = (GC_FINISH) record;

            output.writeByte(0x09); // ADAPT_GC_FINISH

            output.writeInt(r.envId.intValue());
            output.writeLong(r.usedObjects.longValue());
            output.writeLong(r.usedObjectSpace.longValue());
            output.writeLong(r.usedTotalSpace.longValue());
          }
          else if ( record instanceof GC_START )
          {
            GC_START r = (GC_START) record;

            output.writeByte(0x0A); // ADAPT_GC_START

            output.writeInt(r.envId.intValue());
          }
          else if ( record instanceof JVM_INIT_DONE )
          {
            JVM_INIT_DONE r = (JVM_INIT_DONE) record;

            output.writeByte(0x10); // ADAPT_JVM_INIT_DONE

            output.writeInt(r.envId.intValue());
          }
          else if ( record instanceof JVM_SHUT_DOWN )
          {
            JVM_SHUT_DOWN r = (JVM_SHUT_DOWN) record;

            output.writeByte(0x11); // ADAPT_JVM_SHUT_DOWN

            output.writeInt(r.envId.intValue());
          }
          else if ( record instanceof OBJECT_ALLOC )
          {
            OBJECT_ALLOC r = (OBJECT_ALLOC) record;

            output.writeByte(0x1B); // ADAPT_OBJECT_ALLOC

            output.writeInt(r.envId.intValue());
            output.writeInt(r.arenaId.intValue());
            output.writeInt(r.classId.intValue());
            output.writeInt(r.isArray.intValue());
            output.writeInt(r.size.intValue());
            output.writeInt(r.objId.intValue());
          }
          else if ( record instanceof OBJECT_FREE )
          {
            OBJECT_FREE r = (OBJECT_FREE) record;

            output.writeByte(0x1D); // ADAPT_OBJECT_FREE

            output.writeInt(r.envId.intValue());
            output.writeInt(r.objId.intValue());
          }
          else if ( record instanceof METHOD_ENTRY2 )
          {
            METHOD_ENTRY2 r = (METHOD_ENTRY2) record;

            output.writeByte(0x13); // ADAPT_METHOD_ENTRY2

            output.writeInt(r.envId.intValue());
            output.writeInt(r.methodId.intValue());
            output.writeInt(r.objId.intValue());
          }
          else if ( record instanceof METHOD_EXIT )
          {
            METHOD_EXIT r = (METHOD_EXIT) record;

            output.writeByte(0x14); // ADAPT_METHOD_EXIT

            output.writeInt(r.envId.intValue());
            output.writeInt(r.methodId.intValue());
          }
          else if ( record instanceof INSTRUCTION_START )
          {
            INSTRUCTION_START r = (INSTRUCTION_START) record;

            output.writeByte(0x24); // ADAPT_INSTRUCTION_START

            output.writeInt(r.envId.intValue());
            output.writeInt(r.methodId.intValue());
            output.writeInt(r.fieldOffset.intValue());
            output.writeInt(r.fieldIsTrue.intValue());
            output.writeInt(r.fieldKey.intValue());
            output.writeInt(r.fieldLow.intValue());
            output.writeInt(r.fieldHigh.intValue());
            output.writeInt(r.pairIndex.intValue());
            output.writeInt(r.pairTotal.intValue());
          }
          else if ( record == null ) // end of input
          {
            break;
          }
          else
            throw new java.io.IOException("unknown event: "+record);

          count++;

          if ( (count % 10000L) == 0 )
            System.err.print('.');
        }
      }
      finally
      {
        System.err.println();
        System.err.println("decoded "+count+" events");

        stepIn.close();
        output.close();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
}
