/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Obtain serial access to the fields of a record.
 *
 * @see StepRecord#fieldIterator()
 */
public class FieldIterator implements Iterator
{
  private StepObject[] _fields;
  private int          _next;

  /** Create a field iterator from a set of field data. */
  public FieldIterator(StepObject[] fields)
  {
    _fields = fields;
    _next   = 0;
  }

  /** Are there more fields to access? */
  public boolean hasNext()
  {
    return (_next < _fields.length);
  }

  /**
   * Get the next field.
   * @throws NoSuchElementException if there are no more fields.
   */
  public Object next()
  {
    try
    {
      return _fields[_next++];
    }
    catch (ArrayIndexOutOfBoundsException e)
    {
      throw new NoSuchElementException();
    }
  }

  /**
   * <em>Unsupported operation:</em> can't remove fields.
   * @throws UnsupportedOperationException always.
   */
  public void remove()
  {
    throw new UnsupportedOperationException();
  }
}
