/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step;

import step.typedef.*;

/**
 * The common interface for all STEP record objects.
 *
 * <p>
 * In addition to implementing the {@link StepObject} interface and 
 * {@link #fieldIterator()} method, <em>every</em> STEP record class should
 * implement the following features:
 * <ul>
 *   <li> The class should create a definition object ({@link RecordDef}) and
 *        {@linkplain TypeDef#setAsDefault() register} the definition as the
 *        prototype definition for the record.
 *        This operation is normally performed as part of the static (class)
 *        initializer.
 *
 *   <li> The class should provide a {@link RecordFactory} object for creating
 *        new instances from a set of field data. The factory should be added to
 *        the default definition object.
 *        The factory is normally implemented as an inner class of the record.
 * </ul>
 *
 * NOTE: The ordering of serialized fields, in particular with respect to
 * inheritance, is not strictly defined (or enforced). The only constraint is
 * that both the {@link FieldIterator} and {@link RecordFactory} must use the
 * same ordering.
 */
public interface StepRecord extends StepObject
{
  public static final Type TYPE = Type.v("step.StepRecord");
  // -> NOTE: We do other initialization for record types in RecordDef.

  /**
   * Get an iterator for serial access to this record's fields.
   * <p>
   * NOTE: The order in which fields are accessed by the iterator should be
   * compatible with the factory associated with this record.
   *
   * @see RecordFactory
   */
  public FieldIterator fieldIterator();
}
