/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step;

import java.util.List;

import step.typedef.*;

/**
 * Implementation of the built-in STEP <code>string</code> type.
 */
public class StepString implements StepObject
{
  /** The STEP type for <code>string</code> objects. */
  public static final Type TYPE;

  public static final FieldDef LENGTH_FIELD;

  static
  {
    final String name = "StepString";
    final step.typedef.Package pkg = step.typedef.Package.v("step");

    TYPE = Type.v(pkg.qualify(name));
    TYPE.setParent(StepObject.TYPE);
    TYPE.setStepDL("string");

    LENGTH_FIELD = FieldDef.builder().newFieldDef("length", StepInt.TYPE)
      .addAttribute(new Attribute("property", "unsigned"))
      .addAttribute(new Attribute("encoding", "size=creep"))
      .makeFieldDef();

    List fieldDefs = new java.util.ArrayList(1);
    fieldDefs.add(LENGTH_FIELD);
                  
    new TypeDef(name, pkg, fieldDefs, null)
      .setAsDefault();
  }

  private String _str;

  /**
   * Create a new STEP <code>string</code>.
   */
  public StepString(String str)
  {
    _str = str; assert _str != null;
  }

  /**
   * Compare two <code>string</code> objects for equality.
   */
  public boolean equals(Object o) //: StepObject
  {
    try
    {
      return _str.equals(((StepString) o)._str);
    }
    catch (ClassCastException e)
    {
      return false;
    }
    catch (NullPointerException e)
    {
      return false;
    }
  }

  /**
   * Get the hash-code for this object.
   */
  public int hashCode() //: StepObject
  {
    return _str.hashCode();
  }

  /** Get the string value of this object. */
  public String toString()
  {
    return _str;
  }
}
