/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import java.io.IOException;

/**
 * An entity that maintains the incoming stream of data to be decoded.
 * <p>
 * The main purpose of the context is to implement a data stack for unpacking
 * {@link MetaRecord}s that automatically applies {@link MetaEvent}s at the
 * correct time in the decoding proccess. Normally, the context simply defers
 * to some underlying {@link java.io.InputStream} to fill the read requests.
 *
 * @see Encoder
 * @see EncodeContext
 */
public interface DecodeContext
{
  /**
   * Request data from the context.
   *
   * This request is usually a <code>byte[]</code> to be filled, however it may
   * be any encapsulated request agreed upon by the implementation and some
   * {@link Encoder}.
   *
   * @param request an object representing a request for data,
   *   generally a <code>byte[]</code>.
   *
   * @return the filled request.
   *
   * @throws ClassCastException if the type of request cannot be handled.
   *
   * @throws IOException if an I/O error occurred.
   */
  public Object read(Object request) throws IOException;
}
