/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import java.io.IOException;

/**
 * An entity that maintains the outgoing stream of data being encoded.
 * <p>
 * The main purpose of the context is to implement a mechanism for
 * automatically packing data and meta-data into a {@link MetaRecord}.
 *
 * @see Encoder
 * @see DecodeContext
 */
public interface EncodeContext
{
  /**
   * Write a normal data object to the output context.
   *
   * The object is usually a <code>byte[]</code>, however it may
   * be any object agreed upon by the implementation and some
   * {@link Encoder}.
   *
   * @param o the object to write, generally a <code>byte[]</code>.
   *
   * @throws ClassCastException if the context is unable to write objects
   *   of the given type.
   *
   * @throws IOException if the write operation failed.
   */
  public void write(Object o) throws IOException;

  /**
   * Write meta-data at the current position in the output context.
   * @throws IOException if the write operation failed.
   */
  public void metaWrite(MetaEvent e) throws IOException;
}
