/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import java.util.Iterator;

import step.*;
import step.typedef.*;

/**
 * The root of events that are applied to a specific encoder.
 *
 * @see Encoder
 * @see EncoderFactory#queueEvent(EncoderEvent)
 */
public abstract class EncoderEvent implements StepRecord, MetaEvent
{
  public static final RecordDef DEFINITION;

  static
  {
    DEFINITION =
      RecordDef.builder().newRecordDef("EncoderEvent", "step.encoding")
        .addField(FieldDef.builder().newFieldDef("target", StepString.TYPE)
          .addAttribute(new Attribute("encoding", "identifier"))
          .makeFieldDef())
        .makeRecordDef();
  }

  /** The name of the encoder to apply this event to. */
  public StepString target;

  public EncoderEvent(StepString target)
  {
    this.target = target;
  }

  /**
   * Queue this event with the current factory.
   *
   * @see EncoderFactory#queueEvent(EncoderEvent)
   */
  public void apply(EncoderFactory encFactory) //: MetaEvent
  {
    encFactory.queueEvent(this);
  }

  /** Apply this event to a specific encoder. */
  public abstract void applyToEncoder(Encoder encoder);

  /** Get a text representation of this record. */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append(getClass().getName()).append('{');
    Iterator iFields = fieldIterator();
    for (int i=0; iFields.hasNext(); i++)
    {
      if ( i > 0 ) buf.append(',');
      buf.append(iFields.next());
    }
    buf.append('}');

    return buf.toString();
  }
}
