/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import step.*;
import step.typedef.*;

/**
 * An event that dispatches an arbitrary text message to an encoder.
 *
 * @see Encoder#eventMessage(String)
 */
public class EncoderMessageEvent extends EncoderEvent
{
  public static final RecordDef DEFINITION;

  static
  {
    DEFINITION =
      RecordDef.builder().newRecordDef("EncoderMessageEvent", "step.encoding")
        .setParent("step.encoding.EncoderEvent")
        .addField(FieldDef.builder().newFieldDef("message", StepString.TYPE)
          .makeFieldDef())
        .setFactory(new Factory())
        .makeRecordDef();
  }

  /** The message carried by this event. */
  public StepString message;

  public EncoderMessageEvent(StepString target, StepString message)
  {
    super(target);
    this.message = message;
  }

  /**
   * Dispatch the message to the target encoder.
   */
  public void applyToEncoder(Encoder encoder) //: EncoderEvent
  {
    encoder.eventMessage(message.toString());
  }

  /**
   * Compare two events for equality.
   * @throws ClassCastException if the object is not an EncoderMessageEvent
   * @throws NullPointerException if the object is <code>null</code>.
   */
  public boolean equals(Object o) //: StepObject
  {
    EncoderMessageEvent rhs = (EncoderMessageEvent) o;
    return target.equals(rhs.target) && message.equals(message);
  }

  public int hashCode() //: StepObject
  {
    return target.hashCode() + message.hashCode();
  }

  public FieldIterator fieldIterator() //: StepRecord
  {
    StepObject[] fields = { target, message };
    return new FieldIterator(fields);
  }

  public static class Factory implements RecordFactory
  {
    public StepRecord newRecord(StepObject[] fieldData)
    {
      assert fieldData.length == 2;
      return new EncoderMessageEvent(
        (StepString) fieldData[0],
        (StepString) fieldData[1]
      );
    }
  }
}
