/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import step.*;
import step.typedef.*;

/**
 * An event that signals that the next value in the data stream is irregular.
 * The definition of an "irregular" value is specific to the particular encoding
 * strategy.
 *
 * @see RegularValueStrategy
 */
public class IrregularValueEvent extends EncoderEvent
{
  public static final RecordDef DEFINITION;

  static
  {
    DEFINITION =
      RecordDef.builder().newRecordDef("IrregularValueEvent", "step.encoding")
        .setParent("step.encoding.EncoderEvent")
        .setFactory(new Factory())
        .makeRecordDef();
  }

  public IrregularValueEvent(StepString target)
  {
    super(target);
  }

  /**
   * Signal the target encoder that the next value is irregular.
   *
   * @throws ClassCastException if the target encoder is not a
   *   {@link RegularValueStrategy}.
   */
  public void applyToEncoder(Encoder encoder) //: EncoderEvent
  {
    ((RegularValueStrategy) encoder).setNextIsRegular(false);
  }

  /**
   * Compare to events for equality.
   * @throws ClassCastException if the object is not an IrregularValueEvent.
   * @throws NullPointerException if the object is <code>null</code>.
   */
  public boolean equals(Object o) //: StepObject
  {
    IrregularValueEvent rhs = (IrregularValueEvent) o;
    return target.equals(rhs.target);
  }

  public int hashCode() //: StepObject
  {
    return target.hashCode();
  }

  public FieldIterator fieldIterator() //: StepRecord
  {
    StepObject[] fields = { target };
    return new FieldIterator(fields);
  }

  public static class Factory implements RecordFactory
  {
    public StepRecord newRecord(StepObject[] fieldData)
    {
      assert fieldData.length == 1;
      return new IrregularValueEvent((StepString) fieldData[0]);
    }
  }
}
