/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding.profile;

import java.util.Map;
import java.util.Iterator;

import step.*;
import step.util.*;
import step.encoding.*;

/**
 */
public class PCacheStrategy extends CacheStrategy implements Profiled
{
  private long _useCount = 0L;
  private long _hitCount = 0L;

  private Map _valueToCount = new java.util.LinkedHashMap(512, 0.5F);

  public PCacheStrategy(Encoder baseEncoder, int cacheSize)
  {
    super(baseEncoder, cacheSize);
  }

  public void encode(StepObject o, EncodeContext context)
    throws EncoderException, java.io.IOException
  {
    _useCount++;
    if ( _valueToSlot.containsKey(o) ) _hitCount++;

    super.encode(o, context); // do this after to get correct mapping results
  }

  public void printProfile(java.io.PrintWriter out)
  {
    if ( _useCount > 0L )
    {
      StringBuffer buf = new StringBuffer();
      java.text.DecimalFormat fmt = new java.text.DecimalFormat("#0.000%");

      buf.append(getName());
      buf.append("\n\t").append("uses = ").append(_useCount);
      buf.append(", ").append("hits = ").append(_hitCount);

      double uses = (double) _useCount;
      double hits = (double) _hitCount;
      buf.append(" (").append(fmt.format(hits/uses)).append(')');

      out.println(buf);
    }
    else
    {
      out.println(getName()+": UNUSED");
    }
  }
}
