/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.io;

import java.io.ObjectInput;
import java.io.IOException;
import java.io.Serializable;

import step.StepRecord;

/**
 * An intput stream that uses Java's {@linkplain Serializable serialization}
 * mechanism.
 *
 * @see StepRecord
 * @see StepSerialOutput
 */
public class StepSerialInput implements StepRecordInput
{
  private ObjectInput _input;

  /** Create a STEP record input stream from a Java object input stream. */
  public StepSerialInput(ObjectInput input)
  {
    _input = input;
  }

  /**
   * Read the next STEP record from the object input stream.
   *
   * @throws ClassNotFoundException if the class of a serialized object cannot be found.
   * @throws IOException if any of the usual I/O related exceptions occur.
   * @throws ClassCastException if the next object is not a STEP record.
   */
  public StepRecord readRecord() throws IOException, ClassNotFoundException
  {
    return (StepRecord) _input.readObject();
  }

  /** Close this input stream. */
  public void close() throws IOException
  {
    _input.close();
  }
}
