/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.io;

import java.io.ObjectOutput;
import java.io.IOException;
import java.io.Serializable;

import step.StepRecord;

/**
 * An output stream that uses Java's {@linkplain Serializable serialization}
 * mechanism.
 *
 * @see StepRecord
 * @see StepSerialInput
 */
public class StepSerialOutput implements StepRecordOutput
{
  private ObjectOutput _output;

  /** Create a STEP record output stream from a Java object output stream. */
  public StepSerialOutput(ObjectOutput output)
  {
    _output = output;
  }

  /**
   * Write a STEP record to the object output stream.
   *
   * @throws IOException if any of the usual I/O related exceptions occur.
   */
  public void writeRecord(StepRecord record) throws IOException
  {
    _output.writeObject(record);
  }

  /** Flush this output stream. */
  public void flush() throws IOException
  {
    _output.flush();
  }

  /** Close this output stream. */
  public void close() throws IOException
  {
    _output.close();
  }
}
