/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.io;

import java.io.PrintWriter;
import java.io.IOException;

import java.util.Iterator;

import step.*;
import step.typedef.*;

/**
 * Write STEP objects as text, one per line.
 * <p>
 * NOTE: This writer assumes that all written objects (and their fields) have an
 * appropriate definition for <code>toString()</code>.
 *
 * @see StepRecord
 */
public class StepTextOutput implements StepRecordOutput
{
  private PrintWriter _output;

  /**
   * Create a STEP text output stream which prints to the given output stream.
   */
  public StepTextOutput(PrintWriter output)
  {
    assert output != null;

    _output = output;
  }

  /**
   * @throws IOException never.
   *   The underlying {@link PrintWriter} does not throw on writes.
   */
  public void writeRecord(StepRecord record) throws IOException
  {
    assert record != null;

    _output.println(Type.v(record)+":\t"+asText(record));
  }

  public void close() throws IOException
  {
    _output.close();
  }

  public void flush() throws IOException
  {
    _output.flush();
  }

  static String asText(StepRecord record)
  {
    StringBuffer buf = new StringBuffer();

    buf.append('{');

    Iterator iFields    = record.fieldIterator();
    Iterator iFieldDefs = TypeDef.getDefault(record).getFields().iterator();
    for (int i=0; iFields.hasNext(); i++)
    {
      if ( i > 0 ) buf.append(',');

      StepObject field = (StepObject) iFields.next();

      if ( TypeDef.getDefault(field) instanceof RecordDef )
      {
        buf.append(asText((StepRecord) field));
      }
      else // assume field has toString()
      {
        buf.append(field);
      }
    }

    buf.append('}');

    return buf.toString();
  }
}
