/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.typedef;

/**
 * The definition for a field that is inherited from a parent type.
 * <p>
 * NOTE: Inherited fields are a <em>Decorator</em> over the field definition of
 * the parent type.
 *
 * @see LocalFieldDef
 */
public class InheritedFieldDef extends FieldDef
{
  private FieldDef _baseField;
  private boolean  _modified   = false;
  private TypeDef  _altTypeDef = null;

  /** Create the definition for an inherited field. */
  public InheritedFieldDef(FieldDef baseField)
  {
    _baseField = baseField;

    setAttributes(_baseField.getAttributes());

    if ( _baseField.hasAltTypeDef() )
      _altTypeDef = (TypeDef) _baseField.getAltTypeDef().clone();
  }

  public String getName()
  {
    return _baseField.getName();
  }

  /**
   * <em>Illegal operation</em>.
   * It is not possible to set the name of an inherited definition.
   * @throws UnsupportedOperationException always.
   */
  protected void setName(String name)
  {
    throw new UnsupportedOperationException("can't set the name for an inherited field");
  }

  public Type getType()
  {
    return _baseField.getType();
  }

  /**
   * <em>Illegal operation</em>.
   * It is not possible to set the type of an inherited definition.
   * @throws UnsupportedOperationException always.
   */
  protected void setType(Type type)
  {
    throw new UnsupportedOperationException("can't set the type for an inherited field");
  }

  public String getDescription()
  {
    return _baseField.getDescription();
  }

  /**
   * <em>Illegal operation</em>.
   * It is not possible to set the description of an inherited definition.
   * @throws UnsupportedOperationException always.
   */
  protected void setDescription(String description)
  {
    throw new UnsupportedOperationException("can't set the description for an inherited field");
  }

  public TypeDef getAltTypeDef()
  {
    return _altTypeDef;
  }

  protected void setAltTypeDef(TypeDef altDef)
  {
    _altTypeDef = altDef;
  }

  /** Does this definition have modifications over its base definition? */
  public boolean isModified()
  {
    return _modified;
  }

  protected void setModified(boolean modified)
  {
    _modified = modified;
  }

  public Object clone()
  {
    InheritedFieldDef fieldDef = new InheritedFieldDef((FieldDef) _baseField);

    fieldDef.setModified(isModified());

    if ( hasAltTypeDef() )
      fieldDef.setAltTypeDef((TypeDef) getAltTypeDef().clone());

    return fieldDef;
  }

  /** Get the definition that this definition inherits from. */
  protected FieldDef getBaseDefinition()
  {
    return _baseField;
  }
}
