/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.typedef;

import java.util.Iterator;
import java.util.List;

/**
 * The definition of a local (as opposed to inherited) field.
 *
 * @see InheritedFieldDef
 */
public class LocalFieldDef extends FieldDef implements StepDef
{
  private String  _name;
  private Type    _type;
  private String  _description = "";
  private TypeDef _altTypeDef  = null;

  /** Create a new field definition. */
  public LocalFieldDef(String name, Type type)
  {
    setName(name);
    setType(type);
  }

  public String getName()
  {
    return _name;
  }

  protected void setName(String name)
  {
    _name = name; assert _name != null;
  }

  public Type getType()
  {
    return _type;
  }

  protected void setType(Type type)
  {
    _type = type; assert _type != null;
  }

  public String getDescription()
  {
    return _description;
  }

  protected void setDescription(String description)
  {
    _description = description; assert _description != null;
  }

  public TypeDef getAltTypeDef()
  {
    return _altTypeDef;
  }

  protected void setAltTypeDef(TypeDef altDef)
  {
    _altTypeDef = altDef; assert _altTypeDef != null;
  }

  public Object clone()
  {
    FieldDef fieldDef = new LocalFieldDef(getName(), getType());
    fieldDef.setDescription(getDescription());
    fieldDef.setAttributes(getAttributes());

    return fieldDef;
  }

  /** Get this definition as STEP-DL text. */
  public String toStepDL() //: StepDef
  {
    StringBuffer buf = new StringBuffer();

    buf.append(getType().toStepDL()).append(' ').append(getName()).append(' ');

    Iterator iAttributes = getAttributes().iterator();
    while (iAttributes.hasNext())
      buf.append(((Attribute) iAttributes.next()).toStepDL());

    buf.append(';');

    return buf.toString();
  }
}
