/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.util;

/**
 * A utility class to generate string representations of various objects
 * as hexadecimal values.
 *
 * All hex values are prepended with <code>0x</code>.
 * The hex digits are taken from {0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F}.
 */
public class Hex
{
  /** 
   * Get a hex string for an <code>int</code>.
   * @return an 8 digit hex string representing the input value.
   */
  public static String toString(int i)
  {
    String hexString = Integer.toHexString(i).toUpperCase();
    
    while (hexString.length() < 8)
      hexString = "0" + hexString;

    return "0x"+hexString;
  }

  /** 
   * Get a hex string for a <code>long</code>.
   * @return an 16 digit hex string representing the input value.
   */
  public static String toString(long l)
  {
    String hexString = Long.toHexString(l).toUpperCase();
    
    while (hexString.length() < 16)
      hexString = "0" + hexString;

    return "0x"+hexString;
  }

  /** 
   * Get a hex string for a <code>byte</code>.
   * @return a 2 digit hex string representing the input value.
   */
  public static String toString(byte b)
  {
    String hexString = Integer.toHexString(b).toUpperCase();

    while (hexString.length() < 2)
      hexString = "0" + hexString;

    return "0x"+hexString.substring(hexString.length()-2);
  }

  /** 
   * Get a hex string for an array of <code>byte</code>s.
   * @return a comma separated list of bytes as hex values,
   *         surrounded by <code>[</code> and <code>]</code>.
   */
  public static String toString(byte[] bytes)
  {
    if (bytes == null) return "null"; 

    StringBuffer buf = new StringBuffer();

    buf.append('[');
    for (int i=0; i<bytes.length; i++)
    {
      if ( i > 0 ) buf.append(',');
      buf.append(toString(bytes[i]));
    }
    buf.append(']');

    return buf.toString();
  }
}
