/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.util;

/**
 */
// NOTE: Only US-ASCII strings are acceptable.
public class StringConstant
{
  public static final byte BEL       = 0x07;
  public static final byte BS        = 0x08;
  public static final byte HT        = 0x09;
  public static final byte LF        = 0x0A;
  public static final byte VT        = 0x0B;
  public static final byte FF        = 0x0C;
  public static final byte CR        = 0x0D;
  public static final byte QUOTE     = 0x22;
  public static final byte ZERO      = 0x30;
  public static final byte BACKSLASH = 0x5C;
  public static final byte a         = 0x61;
  public static final byte b         = 0x62;
  public static final byte f         = 0x66;
  public static final byte n         = 0x6E;
  public static final byte r         = 0x72;
  public static final byte t         = 0x74;
  public static final byte v         = 0x76;

  private StringConstant() {} // no instantiation

  public static String decode(String str)
  {
    try
    {
      byte[] inBytes = str.getBytes("US-ASCII");

      if ( (inBytes.length < 2) ||
           (inBytes[0] != QUOTE ) ||
           (inBytes[inBytes.length-1] != QUOTE ) )
      {
        throw new RuntimeException("invalid quoted string: "+str); // FIXME
      }

      byte[] outBytes = new byte[inBytes.length];

      int outLen = 0;
      for (int i=1; i<inBytes.length-1; i++)
      {
        if ( inBytes[i] == BACKSLASH )
        {
          if ( (++i) >= (inBytes.length-1) )
            throw new RuntimeException(str+": invalid escape at pos "+i);

          switch ( inBytes[i] )
          {
            case a: outBytes[outLen++] = BEL; break;
            case b: outBytes[outLen++] = BS;  break;
            case t: outBytes[outLen++] = HT;  break;
            case n: outBytes[outLen++] = LF;  break;
            case v: outBytes[outLen++] = VT;  break;
            case f: outBytes[outLen++] = FF;  break;
            case r: outBytes[outLen++] = CR;  break;
            case QUOTE: outBytes[outLen++] = QUOTE; break;
            case BACKSLASH: outBytes[outLen++] = BACKSLASH; break;
            default:
              {
                int code;
                try 
                {
                  code = Integer.parseInt(new String(inBytes, i, 3, "US-ASCII"), 8);
                }
                catch (NumberFormatException e)
                {
                  throw new RuntimeException(str+": invalid escape at pos "+i);
                }

                if ( code > 0x7F )
                  throw new RuntimeException(str+": invalid escape at pos "+i);

                outBytes[outLen++] = (byte) code;
                i += 2;
              }
              break;
          }
        }
        else // normal char
        {
          outBytes[outLen++] = inBytes[i];
        }
      }

      return new String(outBytes, 0, outLen, "US-ASCII");
    }
    catch (java.io.UnsupportedEncodingException e)
    {
      throw new RuntimeException(e);
    }
  }

  public static String encode(String str)
  {
    try
    {
      byte[] inBytes  = str.getBytes("US-ASCII");
      byte[] outBytes = new byte[(4*inBytes.length)+2];

      int outLen = 0;
      outBytes[outLen++] = QUOTE;
      for (int i=0; i<inBytes.length; i++)
      {
        switch ( inBytes[i] )
        {
          case BEL:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = a; break;
          case BS:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = b; break;
          case HT:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = t; break;
          case LF:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = n; break;
          case VT:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = v; break;
          case FF:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = f; break;
          case CR:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = r; break;
          case QUOTE:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = QUOTE; break;
          case BACKSLASH:
            outBytes[outLen++] = BACKSLASH; outBytes[outLen++] = BACKSLASH; break;
          default:
            if ( (inBytes[i] < 0x20) || (inBytes[i] > 0x7E) ) // non-printing
            {
              outBytes[outLen++] = BACKSLASH;
              outBytes[outLen++] = (byte) (ZERO +  (inBytes[i] / 0100));
              outBytes[outLen++] = (byte) (ZERO + ((inBytes[i] % 0100) / 010));
              outBytes[outLen++] = (byte) (ZERO + ((inBytes[i] % 0100) % 010));
            }
            else
            {
              outBytes[outLen++] = inBytes[i];
            }
            break;
        }
      }
      outBytes[outLen++] = QUOTE;

      return new String(outBytes, 0, outLen, "US-ASCII");
    }
    catch (java.io.UnsupportedEncodingException e)
    {
      throw new RuntimeException(e);
    }
  }
}
