/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package stepc;

import java.io.PrintWriter;
  
public class IndentWriter
{
  private int         _level   = 0;
  private boolean     _newline = true;
  private String      _indent  = indentString();
  private String      _lineSep = System.getProperty("line.separator");
  private PrintWriter _out;

  public IndentWriter(PrintWriter out)
  {
    _out = out;
  }

  public void close() { _out.close(); }

  public void flush() { _out.flush(); }

  public void indent()
  {
    _level++;
  }

  public void unindent()
  {
    _level--;

    if ( _level < 0 )
      throw new RuntimeException("cannot have a negative indent level");
  }

  public void print(String str)
  {
    if ( _newline ) for (int i=0; i<_level; i++) _out.print(_indent);

    _out.print(str);
    _newline = false;
  }

  public void println()
  {
    println("");
  }

  public void println(String str)
  {
    if ( _newline ) for (int i=0; i<_level; i++) _out.print(_indent);

    _out.println(str);
    _newline = true;
  }

  static String indentString()
  {
    return System.getProperty("stepc.IndentWriter.indent", "  "/*:default*/);
  }
}
