/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package stepc;

import java.io.PrintWriter;

import java.util.Stack;
import java.util.Iterator;

import step.*;
import step.typedef.*;
import step.util.*;

/**
 * Pretty print resolved STEP definitions.
 */
public class StepdlEmitter implements Emitter
{
  private IndentWriter _output;

  public StepdlEmitter(PrintWriter output)
  {
    _output = new IndentWriter(output);
  }

  public void emit(RecordDef def) // throws IOException
  {
    int pkgScope;
    {
      Stack pkgs = new Stack();
      for (step.typedef.Package pkg = def.getPackage();
           ! pkg.equals(step.typedef.Package.ROOT);
           pkg = pkg.getParent())
      {
        pkgs.add(pkg);
      }
      pkgScope = pkgs.size();

      while (pkgs.size() > 0)
      {
        _output.println("package "+((step.typedef.Package) pkgs.pop()).getName());
        _output.println("{");
        _output.indent();
      }
    }

    _output.print("record "+def.getName());

    if ( ! def.getName().equals(def.getLabel()) )
      _output.print(" "+StringConstant.encode(def.getLabel()));

    if ( def instanceof DerivedRecordDef )
    {
      DerivedRecordDef derivedDef = (DerivedRecordDef) def;
      _output.print(" extends "+
         ((derivedDef.isCleanInherit())?"!":"")+derivedDef.getParent());
    }
    _output.println();

    _output.println("{");
    _output.indent();

    {
      if ( ! def.getDescription().equals("") )
      {
        _output.println(StringConstant.encode(def.getDescription()));
      }

      // record attributes:
      {
        Iterator iAttributes = (def instanceof DerivedRecordDef ) ?
          ((DerivedRecordDef) def).getExtraAttributes().iterator() :
          def.getAttributes().iterator();

        while (iAttributes.hasNext())
          _output.println(((Attribute) iAttributes.next()).toStepDL());
      }

      // field definitions:
      {
        Iterator iFieldDefs = def.getFields().iterator();
        while (iFieldDefs.hasNext())
        {
          FieldDef fieldDef = (FieldDef) iFieldDefs.next();
          if ( fieldDef instanceof LocalFieldDef )
            _output.println(((LocalFieldDef) fieldDef).toStepDL());
        }
      }

      // field modifiers:
      {
        Iterator iModifiers = def.getModifiers().iterator();
        while (iModifiers.hasNext())
        {
          FieldModifier modifier = (FieldModifier) iModifiers.next();

          if ( def.getField(modifier.getTarget()) instanceof LocalFieldDef )
            continue;

          _output.println(modifier.toStepDL());
        }
      }
    }
    _output.unindent();
    _output.println("}");

    for (int j=0; j<pkgScope; j++)
    {
      _output.unindent();
      _output.println("}");
    }

    _output.println();
    _output.flush();
  }

  /**
   * Load record definitions from a classes on the current path and print the
   * definitions as STEP-DL.
   *
   * @param args a list of class names.
   */
  public static void main(String[] args)
  {
    Emitter emitter = new StepdlEmitter(new PrintWriter(System.out, true));

    try
    {
      for (int i=0; i<args.length; i++)
      {
        RecordDef recordDef = RecordDef.load(Type.v(args[i]));
        recordDef.resolve();
        emitter.emit(recordDef);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.exit(1);
    }
  }
}
