package test;

import java.util.List;

import step.*;
import step.typedef.*;
import step.encoding.*;

public class IntTest extends EncTest
{
  public static void main(String[] args)
  {
    StepInt[] pos = {
      new StepInt( (byte)  0x01 ),
      new StepInt( (byte)  0x7F ),
      new StepInt( (short) 0x0001 ),
      new StepInt( (short) 0x00FF ),
      new StepInt( (short) 0x7FFF ),
      new StepInt( (int)   0x00000001 ),
      new StepInt( (int)   0x0000FFFF ),
      new StepInt( (int)   0x7FFFFFFF ),
      new StepInt( (long)  0x0000000000000001L ),
      new StepInt( (long)  0x00000000FFFFFFFFL ),
      new StepInt( (long)  0x7FFFFFFFFFFFFFFFL )
    };

    StepInt[] neg = {
      new StepInt( (byte)  0xFF ),
      new StepInt( (byte)  0x80 ),
      new StepInt( (short) 0xFFFF ),
      new StepInt( (short) 0x80FF ),
      new StepInt( (short) 0x8000 ),
      new StepInt( (int)   0xFFFFFFFF ),
      new StepInt( (int)   0x8000FFFF ),
      new StepInt( (int)   0x80000000 ),
      new StepInt( (long)  0xFFFFFFFFFFFFFFFFL ),
      new StepInt( (long)  0x80000000FFFFFFFFL ),
      new StepInt( (long)  0x8000000000000000L )
    };

    IntTest test = new IntTest();
    List encAttrs = new java.util.LinkedList();

    try
    {
      System.out.println("==== size=creep ====");
      encAttrs.add(new Attribute("encoding", "size=creep"));
      test.doTest(encAttrs, pos);
      test.doTest(encAttrs, neg);

      System.out.println("==== size=1.. ====");
      encAttrs.add(new Attribute("encoding", "size=1..")); // override
      test.doTest(encAttrs, pos);
      test.doTest(encAttrs, neg);

      System.out.println("==== size=1+ ====");
      encAttrs.add(new Attribute("encoding", "size=1+")); // override
      test.doTest(encAttrs, pos);
      test.doTest(encAttrs, neg);

      encAttrs.add(new Attribute("encoding", "unsigned")); // override

      System.out.println("==== unsigned,size=creep ====");
      encAttrs.add(new Attribute("encoding", "size=creep")); // override
      test.doTest(encAttrs, pos);

      System.out.println("==== unsigned,size=1.. ====");
      encAttrs.add(new Attribute("encoding", "size=1..")); // override
      test.doTest(encAttrs, pos);

      System.out.println("==== unsigned,size=1+ ====");
      encAttrs.add(new Attribute("encoding", "size=1+")); // override
      test.doTest(encAttrs, pos);

      ERROR: test.doTest(encAttrs, neg);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  public void doTest(List encAttrs, StepInt[] ints)
    throws EncoderException, java.io.IOException
  {
    Encoder out = getOutFactory().newEncoder("test", StepInt.TYPE, encAttrs);
    Encoder in  = getInFactory().newEncoder("test", StepInt.TYPE, encAttrs);

    for (int i=0; i<ints.length; i++)
    {
      reset();
      System.out.print(ints[i]+" -> ");
      out.encode(ints[i], this);

      reset();
      StepObject tmp = in.decode(this);
      System.out.println(" -> "+tmp+":"+tmp.equals(ints[i]));
    }
  }
}
