package test;

import step.*;
import step.typedef.*;
import step.encoding.*;

public class RecordTest extends EncTest
{
  public static void main(String[] args)
  {
    /* record A {
     *   string x <encoding:"charset=US-ASCII">;
     *   ~x.length <encoding:"size=1">;
     * }
     */
    RecordDef defA = RecordDef.builder().newRecordDef("RecordTest$A", "test")
      .addField(FieldDef.builder().newFieldDef("x", StepString.TYPE)
        .addAttribute(new Attribute("encoding", "charset=US-ASCII"))
        .makeFieldDef())
      .modifyField(new FieldModifier("x.length", false)
        .addAttribute(new Attribute("encoding", "size=1")))
      .setFactory(new A_Factory())
      .makeRecordDef();

    /* record B extends A {
     *   string y;
     *   ~x <encoding:"charset=UTF-8">;
     *   ~x.length <encoding:"size=2">;
     * }
     */
    RecordDef defB = RecordDef.builder().newRecordDef("RecordTest$B", "test")
      .setParent(defA.getFullName(), false)
      .addField(FieldDef.builder().newFieldDef("y", StepString.TYPE)
        .makeFieldDef())
      .modifyField(new FieldModifier("x", false)
        .addAttribute(new Attribute("encoding", "charset=UTF-8")))
      .modifyField(new FieldModifier("x.length", false)
        .addAttribute(new Attribute("encoding", "size=2")))
      .setFactory(new B_Factory())
      .makeRecordDef();

    /* record C extends A {
     *   int y;
     *   ~x <encoding:"charset=UTF-16">;
     *   ~x.length <encoding:"unsigned">;
     * }
     */
    RecordDef defC = RecordDef.builder().newRecordDef("RecordTest$C", "test")
      .setParent(defA.getFullName(), false)
      .addField(FieldDef.builder().newFieldDef("y", StepInt.TYPE)
        .makeFieldDef())
      .modifyField(new FieldModifier("x", false)
        .addAttribute(new Attribute("encoding", "charset=UTF-16")))
      .modifyField(new FieldModifier("x.length", false)
        .addAttribute(new Attribute("encoding", "unsigned")))
      .setFactory(new C_Factory())
      .makeRecordDef();

    /* record D {
     *   A a <g:"D.a">;
     *   ~a.x <encoding:"charset=ISO-8859-1">;
     *   ~a.x.length <encoding:"size=creep">;
     * }
     */
    RecordDef defD = RecordDef.builder().newRecordDef("RecordTest$D", "test")
      .addField(FieldDef.builder().newFieldDef("a", defA.getType())
        .addAttribute(new Attribute("g", "D.a"))
        .makeFieldDef())
      .modifyField(new FieldModifier("a.x", false)
        .addAttribute(new Attribute("encoding", "charset=ISO-8859-1")))
      .modifyField(new FieldModifier("a.x.length", false)
        .addAttribute(new Attribute("encoding", "size=creep")))
      .setFactory(new D_Factory())
      .makeRecordDef();

    StepRecord[] data = 
    {
      new A(new StepString("x1")),
      new A(new StepString("x2")),
      new B(new StepString("x1"), new StepString("Hello")),
      new B(new StepString("x2"), new StepString("World")),
      new C(new StepString("x1"), new StepInt(3)),
      new C(new StepString("x2"), new StepInt(4)),

      new D(new A(new StepString("x1"))),
      new D(new A(new StepString("x2"))),
      new D(new B(new StepString("x1"), new StepString("Hello"))),
      new D(new B(new StepString("x2"), new StepString("World"))),
      new D(new C(new StepString("x1"), new StepInt(3))),
      new D(new C(new StepString("x2"), new StepInt(4)))
    };

    RecordTest test = new RecordTest();

    try
    {
      //defA.resolve();
      //defB.resolve();
      //defC.resolve();
      //defD.resolve();

      //defA.dump(0);
      //defB.dump(0);
      //defC.dump(0);
      //defD.dump(0);

      test.doTest(data);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  public void doTest(StepRecord[] data)
    throws EncoderException, java.io.IOException
  {
    Encoder out = getOutFactory().newRecordEncoder("test");
    Encoder in  = getInFactory().newRecordEncoder("test");
    
    for (int i=0; i<data.length; i++)
    {
      reset();
      System.out.print(data[i]+" -> ");
      out.encode(data[i], this);

      reset();
      StepObject tmp = in.decode(this);
      System.out.println(" -> "+tmp+":"+tmp.equals(data[i]));
    }
  }

  static class A implements StepRecord {
    StepString x;
    public A(StepString x) {
      this.x = x;
    }
    public boolean equals(Object o) {
      return ((A) o).x.equals(x);
    }
    public int hashCode() {
      return x.hashCode();
    }
    public String toString() {
      return "A:{"+x+"}";
    }
    public FieldIterator fieldIterator() {
      StepObject[] fields = { x };
      return new FieldIterator(fields);
    }
  }
  static class A_Factory implements RecordFactory {
    public StepRecord newRecord(StepObject[] fields) {
      return new A((StepString) fields[0]);
    }
  }

  static class B extends A {
    StepString y;
    public B(StepString x, StepString y) {
      super(x);
      this.y = y;
    }
    public boolean equals(Object o) {
      return ((B) o).x.equals(x) && ((B) o).y.equals(y);
    }
    public int hashCode() {
      return x.hashCode() + y.hashCode();
    }
    public String toString() {
      return "B:{"+x+","+y+"}";
    }
    public FieldIterator fieldIterator() {
      StepObject[] fields = { x, y };
      return new FieldIterator(fields);
    }
  }
  static class B_Factory implements RecordFactory {
    public StepRecord newRecord(StepObject[] fields) {
      return new B((StepString) fields[0], (StepString) fields[1]);
    }
  }

  static class C extends A {
    StepInt y;
    public C(StepString x, StepInt y) {
      super(x);
      this.y = y;
    }
    public boolean equals(Object o) {
      return ((C) o).x.equals(x) && ((C) o).y.equals(y);
    }
    public int hashCode() {
      return x.hashCode() + y.hashCode();
    }
    public String toString() {
      return "C:{"+x+","+y+"}";
    }
    public FieldIterator fieldIterator() {
      StepObject[] fields = { x, y };
      return new FieldIterator(fields);
    }
  }
  static class C_Factory implements RecordFactory {
    public StepRecord newRecord(StepObject[] fields) {
      return new C((StepString) fields[0], (StepInt) fields[1]);
    }
  }

  static class D implements StepRecord {
    A a;
    public D(A a) {
      this.a = a;
    }
    public boolean equals(Object o) {
      return ((D) o).a.equals(a);
    }
    public int hashCode() {
      return a.hashCode();
    }
    public String toString() {
      return "D:{"+a+"}";
    }
    public FieldIterator fieldIterator() {
      StepObject[] fields = { a };
      return new FieldIterator(fields);
    }
  }
  static class D_Factory implements RecordFactory {
    public StepRecord newRecord(StepObject[] fields) {
      return new D((A) fields[0]);
    }
  }
}
