/* STEP - An Framework for the Efficient Encoding of Trace Data
 * Copyright (C) 2002 Rhodes Brown
 *
 * STEP is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * STEP is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with STEP; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/* STEP is distributed through the Sable Research Group at:
 * http://www.sable.mcgill.ca/step/
 */

/* Reference Version: $ProjectVersion: 0.9.2.dev.36 $ */

package step.encoding;

import step.StepObject;

/**
 * An encoder that exploits patterns or characteristics of a data sequence.
 */
public abstract class Strategy extends Encoder
{
  protected Encoder _baseEncoder;

  /**
   * Create a strategy with the given base encoder and strategy name.
   */
  protected Strategy(Encoder baseEncoder, String strategyName)
  {
    super(strategyName+"("+baseEncoder.getName()+")", baseEncoder.getFactory());

    _baseEncoder = baseEncoder;
  }

  /**
   * Defer to the underlying base encoder to encode a value.
   */
  protected void baseEncode(StepObject o, EncodeContext context)
    throws EncoderException, java.io.IOException
  {
    _baseEncoder.encode(o, context);
  }

  /**
   * Defer to the underlying base encoder to decode a value.
   */
  protected StepObject baseDecode(DecodeContext context)
    throws EncoderException, java.io.IOException
  {
    return _baseEncoder.decode(context);
  }

  /**
   * Send a message to this encoder.
   * <p>
   * The default behavior is to forward the message to the base encoder.
   */
  protected void eventMessage(String msg)
  {
    _baseEncoder.eventMessage(msg); // => defer to base
  }
}
